'''
 =======================================================================
 ····Y88b···d88P················888b·····d888·d8b·······················
 ·····Y88b·d88P·················8888b···d8888·Y8P·······················
 ······Y88o88P··················88888b·d88888···························
 ·······Y888P··8888b···88888b···888Y88888P888·888·88888b·····d88b·······
 ········888······"88b·888·"88b·888·Y888P·888·888·888·"88b·d88P"88b·····
 ········888···d888888·888··888·888··Y8P··888·888·888··888·888··888·····
 ········888··888··888·888··888·888···"···888·888·888··888·Y88b·888·····
 ········888··"Y888888·888··888·888·······888·888·888··888··"Y88888·····
 ·······························································888·····
 ··························································Y8b·d88P·····
 ···························································"Y88P"······
 =======================================================================

 -----------------------------------------------------------------------
Author       : 焱铭
Date         : 2024-03-03 10:34:41 +0800
LastEditTime : 2024-03-06 12:42:13 +0800
Github       : https://github.com/YanMing-lxb/
FilePath     : /PyTeXMK/src/pytexmk/info_print.py
Description  : 
 -----------------------------------------------------------------------
'''
# -*- coding: utf-8 -*-
import datetime
from rich.console import Console
from rich.table import Table
from rich import box
from rich import print
console = Console(width=100) # 创建控制台对象
# --------------------------------------------------------------------------------
# 定义时间统计函数
# --------------------------------------------------------------------------------
def time_count(fun, *args):
    time_start = datetime.datetime.now()
    fun_return = fun(*args)
    time_end = datetime.datetime.now()
    time_run = time_end - time_start

    time_run = round(time_run.total_seconds(), 4)
    return time_run, fun_return

# --------------------------------------------------------------------------------
# 定义信息打印函数
# --------------------------------------------------------------------------------
def print_message(message):
    padding_size = 80 - (len(message) + sum(1 for c in message if not c.isascii()))-4  # 计算左右两侧 X 的数量
    left_padding = int(padding_size / 2)
    right_padding = padding_size - left_padding
    banner = "[not bold]X[/not bold]" * left_padding + f"| {message} |" + "[not bold]X[/not bold]" * right_padding
    console.print("\n\n" + "=" * 80, style="yellow bold")
    console.print(banner, style="red on white bold")
    console.print("=" * 80 + "\n\n", style="yellow bold")
    

# --------------------------------------------------------------------------------
# 定义统计时间打印函数
# --------------------------------------------------------------------------------
def time_print(start_time, name_target_list, time_run_list):
    end_time = datetime.datetime.now() # 计算结束时间
    run_time = end_time - start_time
    hours, remainder = divmod(run_time.seconds, 3600)
    minutes, seconds = divmod(remainder, 60)
    milliseconds = run_time.microseconds // 1000  # 获取毫秒部分

    number_programmes_run = len(name_target_list)-4
    
    time_compile = sum(time_run_list)
    name_target_list.append('LaTeX 编译时长')
    time_run_list.append(time_compile)

    time_other_operating = float(run_time.total_seconds()) - time_compile
    name_target_list.append('Python 运行时长')
    time_run_list.append(time_other_operating)

    time_pytexmk = float(run_time.total_seconds())
    name_target_list.append('PyTeXMK 运行时长')
    time_run_list.append(time_pytexmk)

    # 创建表格对象
    table = Table(show_header=True, header_style="bold magenta", box=box.ASCII_DOUBLE_HEAD, 
                title="PyTeXMK 运行时长统计表")

    # 定义列名
    table.add_column("序号", justify="center", no_wrap=True)
    table.add_column("运行项目", style="cyan", justify="center", no_wrap=True)
    table.add_column("运行时长", style="green", justify="center", no_wrap=True)
    table.add_column("序号", justify="center", no_wrap=True)
    table.add_column("运行项目", style="cyan", justify="center")
    table.add_column("运行时长", style="green", justify="center", no_wrap=True)

    # 判断统计项目列数是否是偶数
    length = len(name_target_list)/2 # 计算打印表格列数
    row_num = None

    # 判断统计项目列数是否是偶数
    if length - int(length) < 0.5:
        row_num = int(length)
    else: # 是偶数则加一
        row_num = int(length) + 1 

    # 添加数据到表格
    for i in range(row_num):
        table.add_row(
            str(i+1),
            name_target_list[i],
            "{:.4f} s".format(time_run_list[i]),
            str(i+1+row_num) if i+row_num <= len(name_target_list) else "",
            name_target_list[i+row_num] if i+row_num < len(name_target_list) else "",
            "{:.4f} s".format(time_run_list[i+row_num]) if i+row_num < len(name_target_list) else ""  
        )

    print("\n" + "=" * 80 + "\n")
    console.print(table) # 打印表格

    print(f"PyTeXMK 运行时长：{hours} 小时 {minutes} 分 {seconds} 秒 {milliseconds} 毫秒 ({run_time.total_seconds():.3f} s total)")
    print(f"运行函数：{number_programmes_run} 个")


'''
 =======================================================================
 ····Y88b···d88P················888b·····d888·d8b·······················
 ·····Y88b·d88P·················8888b···d8888·Y8P·······················
 ······Y88o88P··················88888b·d88888···························
 ·······Y888P··8888b···88888b···888Y88888P888·888·88888b·····d88b·······
 ········888······"88b·888·"88b·888·Y888P·888·888·888·"88b·d88P"88b·····
 ········888···d888888·888··888·888··Y8P··888·888·888··888·888··888·····
 ········888··888··888·888··888·888···"···888·888·888··888·Y88b·888·····
 ········888··"Y888888·888··888·888·······888·888·888··888··"Y88888·····
 ·······························································888·····
 ··························································Y8b·d88P·····
 ···························································"Y88P"······
 =======================================================================

 -----------------------------------------------------------------------
Author       : 焱铭
Date         : 2024-02-29 15:43:26 +0800
LastEditTime : 2024-03-16 20:17:45 +0800
Github       : https://github.com/YanMing-lxb/
FilePath     : /PyTeXMK/src/pytexmk/compile_model.py
Description  : 
 -----------------------------------------------------------------------
'''
# -*- coding: utf-8 -*-
import os
import re
import subprocess
from .info_print import print_message
from rich import console
console = console.Console(width=100)  # 设置宽度为80

# --------------------------------------------------------------------------------
# 定义编译 TeX 文件命令
# --------------------------------------------------------------------------------
def compile_tex(tex_name, file_name, tex_times, quiet):
    options = [tex_name, "-shell-escape", "-file-line-error", "-halt-on-error", "-synctex=1", f'{file_name}.tex']
    print_message(f"{tex_times} 次 {tex_name} 编译")
    if tex_name == 'xelatex':
        options.insert(5, "-no-pdf")
    if quiet:
        options.insert(4, "-interaction=batchmode") # 静默编译
    else:
        options.insert(4, "-interaction=nonstopmode") # 非静默编译
    console.print(f"[bold]运行命令：[/bold][red][cyan]{' '.join(options)}[/cyan][/red]\n")
    subprocess.run(options)

# --------------------------------------------------------------------------------
# 定义编译参考文献命令
# --------------------------------------------------------------------------------
def compile_bib(file_name, quiet):
    if os.path.exists(f"{file_name}.aux"):
        with open(f"{file_name}.aux", 'r', encoding='utf-8') as aux_file:
            aux_content = aux_file.read()

        if re.search(r'\\bibdata|\\abx@aux@cite', aux_content):
            if re.search(r'\\abx@aux@refcontext', aux_content):
                name_target = "biber 编译"
                print_message('biber 文献编译')
                options = ["biber", file_name]
                if quiet:
                    options.insert(1, "-quiet") # 静默编译
                console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
                subprocess.run(options)
            elif re.search(r'\\bibdata', aux_content):
                name_target = 'bibtex 编译'
                print_message('bibtex 文献编译')
                options = ['bibtex', file_name]
                console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
                subprocess.run(options)
            compile_tex_times = 2 # 参考文献需要额外编译的次数
            print_bib = f"{name_target}参考文献"
        elif re.search(r'\\bibcite', aux_content):
            compile_tex_times = 1
            name_target = None
            print_bib = "thebibliography 环境实现排版 "
        else:
            compile_tex_times = 0
            name_target = None
            print_bib = "没有引用参考文献或编译工具不属于 bibtex 或 biber "
    else:
        compile_tex_times = 0
        name_target = None
        print_bib = "文档没有参考文献"
    return compile_tex_times, print_bib, name_target

# --------------------------------------------------------------------------------
# 定义编译目录索引命令
# --------------------------------------------------------------------------------
def compile_index(file_name):
    if any(os.path.exists(f"{file_name}{ext}") for ext in [".glo", ".nlo", ".idx", ".toc"]):
        if os.path.exists(f"{file_name}.glo"):
            with open(f"{file_name}.glo", "r", encoding='utf-8') as f:
                content = f.read()
            if content.strip():  # Check if content is not empty
                compile_tex_times = 1 # 目录和符号索引需要额外编译的次数
                name_target = "glossaries 宏包"
                print_message("glossaries 宏包")
                print_index = "glossaries 宏包生成符号索引"
                print(print_index,"\n")
                options = ["makeindex", "-s", f"{file_name}.ist", "-o", f"{file_name}.gls", f"{file_name}.glo"]
                console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
                subprocess.run(options)
            else:
                compile_tex_times = 0
                name_target = None
                print_index = "glossaries 宏包没有进行索引"

        elif os.path.exists(f"{file_name}.nlo"):
            with open(f"{file_name}.nlo", "r", encoding='utf-8') as f:
                content = f.read()
            if content.strip():  # Check if content is not empty
                compile_tex_times = 1
                name_target = "nomencl 宏包"
                print_message("nomencl 宏包")
                print_index = "nomencl 宏包生成符号索引"
                print(print_index,"\n")
                options = ["makeindex", "-s", "nomencl.ist", "-o", f"{file_name}.nls", f"{file_name}.nlo"]
                console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
                subprocess.run(options)
            else:
                compile_tex_times = 0
                name_target = None
                print_index = "nomencl 宏包没有进行索引"
        elif os.path.exists(f"{file_name}.idx"):
            with open(f"{file_name}.idx", "r", encoding='utf-8') as f:
                content = f.read()
            if content.strip():  # Check if content is not empty
                compile_tex_times = 1
                name_target = "makeidx 宏包"
                print_message("makeidx 宏包")
                print_index = "makeidx 宏包生成索引"
                print(print_index,"\n")
                options = ["makeindex", f"{file_name}.idx"]
                console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
                subprocess.run(options)
            else:
                compile_tex_times = 0
                name_target = None
                print_index = "makeidx 宏包没有进行索引"  

        else:
            name_target = None
            if os.path.exists(f"{file_name}.toc"):
                compile_tex_times = 1 # 目录需要额外编译 1 次
                print_index = "含有图/表/章节目录"
            else:
                compile_tex_times = 0
                print_index = "没有图/表/章节目录"

    else: 
        compile_tex_times = 0
        name_target = None
        print_index = "没有插入任何目录或者使用 makeidx、glossaries、nomencl 等宏包"

    return compile_tex_times, print_index, name_target

# --------------------------------------------------------------------------------
# 定义编译 xdv 文件命令
# --------------------------------------------------------------------------------
def compile_xdv(file_name, quiet):
    print_message("dvipdfmx 编译")
    options = ["dvipdfmx", "-V", "2.0", f"{file_name}"]
    if quiet:
        options.insert(1, "-q") # 静默编译
    console.print(f"[bold]运行命令：[/bold][cyan]{' '.join(options)}[/cyan]\n")
    subprocess.run(options) # 将 xelatex 生成的 xdv 转换成 pdf


'''
 =======================================================================
 ····Y88b···d88P················888b·····d888·d8b·······················
 ·····Y88b·d88P·················8888b···d8888·Y8P·······················
 ······Y88o88P··················88888b·d88888···························
 ·······Y888P··8888b···88888b···888Y88888P888·888·88888b·····d88b·······
 ········888······"88b·888·"88b·888·Y888P·888·888·888·"88b·d88P"88b·····
 ········888···d888888·888··888·888··Y8P··888·888·888··888·888··888·····
 ········888··888··888·888··888·888···"···888·888·888··888·Y88b·888·····
 ········888··"Y888888·888··888·888·······888·888·888··888··"Y88888·····
 ·······························································888·····
 ··························································Y8b·d88P·····
 ···························································"Y88P"······
 =======================================================================

 -----------------------------------------------------------------------
Author       : 焱铭
Date         : 2024-02-28 23:11:52 +0800
LastEditTime : 2024-03-22 10:35:41 +0800
Github       : https://github.com/YanMing-lxb/
FilePath     : /PyTeXMK/src/pytexmk/__main__.py
Description  : 
 -----------------------------------------------------------------------
'''
# -*- coding: utf-8 -*-
import argparse
import datetime
from .version import script_name, __version__
from .compile_model import compile_tex, compile_bib, compile_index, compile_xdv
from .additional_operation import remove_aux, remove_result, remove_result_in_root, move_result, search_file, check_file_name, clean_all_pdf
from .info_print import time_count, time_print, print_message

# ================================================================================
# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX 整体进行编译 XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
# ================================================================================
def compile(tex_name, file_name, quiet, build_path):
    name_target_list = []
    time_run_list = []

    time_run_remove_aux, _ = time_count(remove_aux, file_name) # 清除已有辅助文件
    name_target_list.append("清除旧辅助文件")
    time_run_list.append(time_run_remove_aux)

    time_run_tex, _ = time_count(compile_tex, tex_name, file_name, 1, quiet) # 首次编译 tex 文档
    time_run_bib, return_com_bib = time_count(compile_bib, file_name, quiet) # 编译参考文献
    time_run_index, return_com_index = time_count(compile_index, file_name) # 编译目录索引
    
    times_compile_tex_bib, print_bib, name_target_bib = return_com_bib # 获取 compile_bib 函数得到的参数
    times_compile_tex_index, print_index, name_target_index = return_com_index # 获取 compile_index 函数得到的参数
    times_extra_complie = max(times_compile_tex_bib, times_compile_tex_index) # 计算额外编译 tex 文档次数

    # 将获取到的编译项目名称 添加到对应的列表中
    name_target_list.append(f'{tex_name} 1st')
    time_run_list.append(time_run_tex)
    
    if times_compile_tex_bib != 0: # 存在参考文献编译过程
        if name_target_bib:
            name_target_list.append(name_target_bib)
            time_run_list.append(time_run_bib)
    if times_compile_tex_index != 0: # 存在目录索引编译过程
        if name_target_index:
            name_target_list.append(name_target_index)
            time_run_list.append(time_run_index)

    for i in range(times_extra_complie): # 进行额外编译 tex
        time_run_tex, _ = time_count(compile_tex, tex_name, file_name, i + 2, quiet)
        if i+2 == 2: 
            name_target_list.append(f'{tex_name} {i+2}nd')
        elif i+2 == 3:
            name_target_list.append(f'{tex_name} {i+2}rd')

        time_run_list.append(time_run_tex)

    if tex_name == "xelatex":  # 判断是否编译 xdv 文件
        time_run_xdv, _ = time_count(compile_xdv, file_name, quiet) # 编译 xdv 文件
        name_target_list.append('dvipdfmx 编译')
        time_run_list.append(time_run_xdv)

    print("\n\n" + "=" * 80 + "\n" +
          "▓" * 33 + " 完成所有编译 " + "▓" * 33 + "\n" +
          "=" * 80 + "\n")
    print(f"文档整体：{tex_name} 编译 {times_extra_complie+1} 次")
    print(f"参考文献：{print_bib}")
    print(f"目录索引：{print_index}")
    print_message("开始执行编译以外的附加命令！")
    
    time_run_remove_res, _ = time_count(remove_result, build_path) # 清除已有结果文件
    name_target_list.append("清除旧结果文件")
    time_run_list.append(time_run_remove_res)
    time_run_move_res, _ = time_count(move_result, file_name, build_path) # 移动生成结果文件
    name_target_list.append("移动结果文件")
    time_run_list.append(time_run_move_res)
    time_run_remove_aux, _ = time_count(remove_aux, file_name) # 清除生成辅助文件
    name_target_list.append("清除辅助文件")
    time_run_list.append(time_run_remove_aux)

    return name_target_list, time_run_list

def main():
    # ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓ 设置默认 ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓
    tex_name = "xelatex"
    build_path = "./Build/"
    # ▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓▓

    start_time = datetime.datetime.now() # 计算开始时间
    
    # --------------------------------------------------------------------------------
    # 定义命令行参数
    # --------------------------------------------------------------------------------
    parser = argparse.ArgumentParser(description="LaTeX 辅助编译程序.")
    parser.add_argument('-v', '--version', action='version', version=f'{script_name}: {__version__}')
    parser.add_argument('-p', '--pdflatex', action='store_true', help="pdflatex 进行编译")
    parser.add_argument('-x', '--xelatex', action='store_true', help="xelatex 进行编译")
    parser.add_argument('-l', '--lualatex', action='store_true', help="lualatex 进行编译")
    parser.add_argument('-c', '--clean', action='store_true', help="清除所有辅助文件")
    parser.add_argument('-C', '--Clean', action='store_true', help="清除所有辅助文件和 pdf 文件")
    parser.add_argument('-nq', '--no-quiet', action='store_true', help="非安静模式运行，此模式下显示编译过程")
    parser.add_argument('-cp', '--clean-pdf', action='store_true', help="清理 pdf 文件，当 LaTeX 编译过程中警告 invalid X X R object 时，可使用此参数清理所有 pdf 文件")
    
    parser.add_argument('document', nargs='?', help="要被编译的文件名")
    args = parser.parse_args()

    if args.xelatex:
        tex_name = "xelatex"
    if args.pdflatex:
        tex_name = "pdflatex"
    if args.lualatex:
        tex_name = "lualatex"
        
    if args.document: # 指定 latex 文件
        file_name = check_file_name(args.document) # check_file_name 函数检查 args.document 参数输入的文件名是否正确
    else: # 未指定 latex 文件
        file_name = search_file() # 运行 search_file 函数判断

    if file_name: # 如果存在 file_name
        if args.clean:
            remove_aux(file_name)
        elif args.Clean:
            remove_aux(file_name)
            remove_result(build_path,build_path,file_name)
            remove_result_in_root(file_name)
        elif args.clean_pdf:
            clean_all_pdf('.')
        else:
            name_target_list, time_run_list = compile(tex_name, file_name, not args.no_quiet, build_path)
            time_print(start_time, name_target_list, time_run_list) # 打印编译时长统计
    

if __name__ == "__main__":

    main()


import os
import shutil
import datetime
import argparse
import subprocess
from rich.console import Console
from rich.table import Table

def copy(source_folder, destination_folder):
    # 确保目标文件夹存在，如果不存在则创建
    if not os.path.exists(destination_folder):
        os.makedirs(destination_folder)

    # 遍历源文件夹中的所有文件
    for filename in os.listdir(source_folder):
        source_file = os.path.join(source_folder, filename)
        destination_file = os.path.join(destination_folder, filename)

        # 判断源文件是否是文件
        if os.path.isfile(source_file):
            # 复制文件到目标文件夹
            shutil.copy(source_file, destination_file)

# 修改函数为可运行函数
def file_modify(destination_folder):
    # 读取原始文件内容
    for root, dirs, files in os.walk(destination_folder):
        for file_name in files:
            if file_name.endswith('.py'):
                file_path = os.path.join(root, file_name)
                with open(file_path, 'r', encoding='utf-8') as file:
                    content = file.read()
                content = content.replace('from .', 'from ') # 替换 from . 为 from
                with open(file_path, 'w', encoding='utf-8') as file: 
                    file.write(content) # 写入更新后的内容到同一文件

# --------------------------------------------------------------------------------
# 测试函数
# --------------------------------------------------------------------------------
def test(test_files, test_file_type, command, run_command, destination_folder):
    # 存储所有要打印的内容
    print_output = []
    # 测试 biblatex, bibtex, 图目录表目录目录, 目录, glossaries, nomencl
    for i in range(len(test_files)):
        try:
            time_start = datetime.datetime.now()
            result = subprocess.run(run_command + [test_files[i]], cwd=destination_folder)
            time_end = datetime.datetime.now()
            time_run = round((time_end - time_start).total_seconds(), 4)
            if result.returncode == 0:
                print_output.append([test_file_type[i], time_run, "[green]通过"])  # 绿色
            else:
                print_output.append([test_file_type[i], time_run, "[red]未通过"])  # 红色
        except Exception as e:
            print_output.append([test_file_type[i], 0, "[red]未通过"])  # 红色
    # 测试 pytexmk 参数
    for i in range(len(command)):
        try:
            if command[i] == "-nq":
                time_start = datetime.datetime.now()
                result = subprocess.run(run_command + ['-nq', 'main'], cwd=destination_folder)
                time_end = datetime.datetime.now()
                time_run = round((time_end - time_start).total_seconds(), 4)
                if result.returncode == 0:
                    print_output.append([command[i], time_run, "[green]通过"])  # 绿色
                else:
                    print_output.append([command[i], time_run, "[red]未通过"])  
            elif command[i] == "-c" and "-C":
                time_start = datetime.datetime.now()
                subprocess.run(['xelatex', "-shell-escape", "-file-line-error", "-halt-on-error", "-interaction=batchmode", 'main'], cwd=destination_folder)
                result = subprocess.run(run_command + [command[i]], cwd=destination_folder)
                time_end = datetime.datetime.now()
                time_run = round((time_end - time_start).total_seconds(), 4)
                if result.returncode == 0:
                    print_output.append([command[i], time_run, "[green]通过"])  # 绿色
                else:
                    print_output.append([command[i], time_run, "[red]未通过"])  # 红色
            else:
                time_start = datetime.datetime.now()
                result = subprocess.run(run_command + [command[i]], cwd=destination_folder)
                time_end = datetime.datetime.now()
                time_run = round((time_end - time_start).total_seconds(), 4)
                if result.returncode == 0:
                    print_output.append([command[i], time_run, "[green]通过"])  # 绿色
                else:
                    print_output.append([command[i], time_run, "[red]未通过"])  # 红色
        except Exception as e:
            print_output.append([i, 0, "[red]未通过"])  # 红色
    return print_output

# --------------------------------------------------------------------------------
# 清除临时测试文件夹函数
# --------------------------------------------------------------------------------
def remove(path):
    if os.path.exists(path):
        shutil.rmtree(path)  # 删除整个文件夹
        print("删除临时测试文件夹\n")

# --------------------------------------------------------------------------------
# 测试统计表打印函数
# --------------------------------------------------------------------------------
def print_table(data):
    console = Console() # 创建Console对象

    table = Table(show_header=True, header_style="bold magenta", 
                title="PyTeXMK 测试统计表")

    # 定义列名
    table.add_column("序号", style="yellow", justify="center", no_wrap=True)
    table.add_column("测试项目", style="cyan", justify="left", no_wrap=True)
    table.add_column("测试时长", style="green", justify="left", no_wrap=True)
    table.add_column("状态", justify="center", no_wrap=True)
    table.add_column("序号", style="yellow", justify="center", no_wrap=True)
    table.add_column("测试项目", style="cyan", justify="left")
    table.add_column("测试时长", style="green", justify="left", no_wrap=True)
    table.add_column("状态", justify="center", no_wrap=True)

    # 判断统计项目列数是否是偶数
    length = len(data)/2 # 计算打印表格列数
    row_num = ""
    # 判断统计项目列数是否是偶数
    if length - int(length) < 0.5:
        row_num = int(length)
    else: # 是偶数则加一
        row_num = int(length) + 1

    # 添加数据到表格
    for i in range(row_num):
        table.add_row(
            str(i+1),
            data[i][0],
            "{:.4f} s".format(data[i][1]),
            data[i][2],
            str(i+1+row_num) if i+row_num <= len(data) else "",
            data[i+row_num][0] if i+row_num < len(data) else "",
            "{:.4f} s".format(data[i+row_num][1]) if i+row_num < len(data) else "",
            data[i+row_num][2] if i+row_num < len(data) else ""
        )

    console.print(table) # 打印表格

# --------------------------------------------------------------------------------
# 测试目标
# --------------------------------------------------------------------------------
parser = argparse.ArgumentParser(description="辅助测试文件.")
parser.add_argument('-v', '--version', action='version', version='test: v 0.1')
parser.add_argument('-w', '--whl', action='store_true', help="测试 whl 文件")
args = parser.parse_args()


test_files = [
    "biblatex-test",
    "bibtex-test",
    "thebibliography-test",
    "contents-figrue-table-test",
    "contents-test",
    "glossaries-test",
    "nomencl-test",
    "makeidx-test"
]
test_file_type = ['biblatex', 'bibtex', 'thebibliography', '图表目录', '目录', 'glossaries', 'nomencl', "makeidx"]
command = ['-v', '-h', '-nq', '-c', '-C']

source_folder = 'src/pytexmk' # 源文件夹路径
tests_folder = 'tests' # 测试文件路径
destination_folder = 'test-temp' # 目标文件夹路径
run_file = "__main__.py"
run_command = ["python", "__main__.py"]
if args.whl:
    run_command = ["pytexmk"]

# --------------------------------------------------------------------------------
# 运行测试流程
# --------------------------------------------------------------------------------
start_time = datetime.datetime.now() # 计算开始时间
remove(destination_folder) # 删除临时测试文件夹
copy(source_folder, destination_folder) # 复制测试对象到目标位置
copy(tests_folder, destination_folder) # 复制测试文件到目标位置
file_modify(destination_folder) # 修改测试对象使其可运行
data = test(test_files, test_file_type, command, run_command, destination_folder)
remove(destination_folder) # 删除临时测试文件夹
print_table(data)
# subprocess.run([run_command, test_file, '-C'], cwd=destination_folder)

# --------------------------------------------------------------------------------
# 打印统计s
# --------------------------------------------------------------------------------
end_time = datetime.datetime.now() # 计算结束时间
run_time = end_time - start_time
hours, remainder = divmod(run_time.seconds, 3600)
minutes, seconds = divmod(remainder, 60)
milliseconds = run_time.microseconds // 1000  # 获取毫秒部分
print("\n" + "=" * 80)
print(f"测试时长为：{hours} 小时 {minutes} 分 {seconds} 秒 {milliseconds} 毫秒 ({run_time.total_seconds():.3f} s total)\n")